/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (c) 2024 Rockchip Electronics Co., Ltd.
 * Author: Finley Xiao <finley.xiao@rock-chips.com>
 */

#ifndef __DT_BINDINGS_POWER_RV1126B_POWER_CONTROLLER_H__
#define __DT_BINDINGS_POWER_RV1126B_POWER_CONTROLLER_H__

/* VD_NPU */
#define RV1126B_PD_NPU		0

/* VD_LOGIC */
#define RV1126B_PD_VDO		1
#define RV1126B_PD_AIISP	2

#endif
