/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * linux/sound/rt1015.h -- Platform data for RT1015
 *
 * Copyright 2020 Realtek Microelectronics
 */

#ifndef __LINUX_SND_RT1015_H
#define __LINUX_SND_RT1015_H

struct rt1015_platform_data {
	unsigned int power_up_delay_ms;
};

#endif
